"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebsocketConsumersStreamWorker = void 0;
const redis_smq_1 = require("redis-smq");
const redis_smq_common_1 = require("redis-smq-common");
class WebsocketConsumersStreamWorker extends redis_smq_common_1.Worker {
    constructor(redisClient, queueManager, managed) {
        super(managed);
        this.consumersHeartbeats = {};
        this.onlineConsumers = [];
        this.handleQueueConsumers = (queue, cb) => {
            redis_smq_1.Consumer.getQueueConsumers(this.redisClient, queue, false, (err, reply) => {
                if (err)
                    cb(err);
                else {
                    const queueConsumers = reply !== null && reply !== void 0 ? reply : {};
                    this.redisClient.publish(`streamQueueConsumers:${queue.ns}:${queue.name}`, JSON.stringify(queueConsumers), () => cb(null, queueConsumers));
                }
            });
        };
        this.handleQueueConsumersHeartbeats = (queue, queueConsumers, cb) => {
            const queueOnlineConsumers = [];
            redis_smq_common_1.async.eachIn(queueConsumers, (consumerInfo, consumerId, done) => {
                if (this.consumersHeartbeats.hasOwnProperty(consumerId)) {
                    if (this.consumersHeartbeats[consumerId])
                        queueOnlineConsumers.push(consumerId);
                    done();
                }
                else {
                    redis_smq_1.Consumer.getConsumerHeartbeat(this.redisClient, consumerId, (err, heartbeat) => {
                        if (err)
                            done(err);
                        else if (heartbeat) {
                            this.consumersHeartbeats[consumerId] = heartbeat;
                            this.onlineConsumers.push(consumerId);
                            queueOnlineConsumers.push(consumerId);
                            this.redisClient.publish(`streamConsumerHeartbeat:${consumerId}`, JSON.stringify(heartbeat), () => done());
                        }
                        else {
                            this.consumersHeartbeats[consumerId] = false;
                            done();
                        }
                    });
                }
            }, (err) => {
                if (err)
                    cb(err);
                else {
                    this.redisClient.publish(`streamQueueOnlineConsumers:${queue.ns}:${queue.name}`, JSON.stringify({ ids: queueOnlineConsumers }), () => cb());
                }
            });
        };
        this.work = (cb) => {
            this.consumersHeartbeats = {};
            this.onlineConsumers = [];
            redis_smq_common_1.async.waterfall([
                (cb) => this.queueManager.queue.list(cb),
                (queues, cb) => {
                    redis_smq_common_1.async.eachOf(queues, (queue, _, done) => {
                        redis_smq_common_1.async.waterfall([
                            (cb) => {
                                this.handleQueueConsumers(queue, cb);
                            },
                            (queueConsumers, cb) => {
                                this.handleQueueConsumersHeartbeats(queue, queueConsumers, cb);
                            },
                            (cb) => {
                                this.redisClient.publish(`streamOnlineConsumers`, JSON.stringify({ ids: this.onlineConsumers }), () => cb());
                            },
                        ], done);
                    }, cb);
                },
            ], (err) => cb(err));
        };
        this.queueManager = queueManager;
        this.redisClient = redisClient;
    }
}
exports.WebsocketConsumersStreamWorker = WebsocketConsumersStreamWorker;
exports.default = WebsocketConsumersStreamWorker;
//# sourceMappingURL=websocket-consumers-stream.worker.js.map